@@ -33,12 +33,63 @@ namespace GDBusCXX {
 MethodHandler::MethodMap MethodHandler::m_methodMap;
 boost::function<void (void)> MethodHandler::m_callback;
 
-static void GDBusNameLost(GDBusConnection *connection,
-                          const gchar *name,
-                          gpointer user_data)
+// It should be okay to use global variables here because they are
+// only used inside the main thread while it waits in undelay() for a
+// positive or negative result to g_bus_own_name_on_connection().
+// Once acquired, the name can only get lost again when the
+// D-Bus daemon dies (no name owership alloed), in which case the
+// process dies anyway.
+static bool nameError;
+static bool nameAcquired;
+static void BusNameAcquired(GDBusConnection *connection,
+                            const gchar *name,
+                            gpointer userData) throw ()
 {
-    g_critical("lost D-Bus connection or failed to obtain %s D-Bus name, quitting", name);
-    exit(1);
+    try {
+        g_debug("got D-Bus name %s", name);
+        nameAcquired = true;
+    } catch (...) {
+        nameError = true;
+    }
+}
+
+static void BusNameLost(GDBusConnection *connection,
+                        const gchar *name,
+                        gpointer userData) throw ()
+{
+    try {
+        g_debug("lost %s %s",
+                connection ? "D-Bus connection for name" :
+                "D-Bus name",
+                name);
+    } catch (...) {
+    }
+    nameError = true;
+}
+
+void DBusConnectionPtr::undelay() const
+{
+    if (!m_name.empty()) {
+        g_debug("starting to acquire D-Bus name %s", m_name.c_str());
+        nameAcquired = false;
+        nameError = false;
+        char *copy = g_strdup(m_name.c_str());
+        g_bus_own_name_on_connection(get(),
+                                     copy,
+                                     G_BUS_NAME_OWNER_FLAGS_NONE,
+                                     BusNameAcquired,
+                                     BusNameLost,
+                                     copy,
+                                     g_free);
+        while (!nameAcquired && !nameError) {
+            g_main_context_iteration(NULL, true);
+        }
+        g_debug("done with acquisition of %s", m_name.c_str());
+        if (nameError) {
+            throw std::runtime_error("could not obtain D-Bus name - already running?");
+        }
+    }
+    g_dbus_connection_start_message_processing(get());
 }
 
 DBusConnectionPtr dbus_get_bus_connection(const char *busType,
@@ -48,10 +99,13 @@ DBusConnectionPtr dbus_get_bus_connection(const char *busType,
 {
     DBusConnectionPtr conn;
     GError* error = NULL;
+    GBusType type =
+        boost::iequals(busType, "SESSION") ?
+        G_BUS_TYPE_SESSION :
+        G_BUS_TYPE_SYSTEM;
 
-    if(unshared) {
-        char *address = g_dbus_address_get_for_bus_sync(boost::iequals(busType, "SESSION") ?
-                                                        G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM,
+    if (unshared) {
+        char *address = g_dbus_address_get_for_bus_sync(type,
                                                         NULL, &error);
         if(address == NULL) {
             if (err) {
@@ -76,9 +130,7 @@ DBusConnectionPtr dbus_get_bus_connection(const char *busType,
         }
     } else {
         // This returns a singleton, shared connection object.
-        conn = DBusConnectionPtr(g_bus_get_sync(boost::iequals(busType, "SESSION") ?
-                                                G_BUS_TYPE_SESSION :
-                                                G_BUS_TYPE_SYSTEM,
+        conn = DBusConnectionPtr(g_bus_get_sync(type,
                                                 NULL, &error),
                                  false);
         if(conn == NULL) {
@@ -89,11 +141,11 @@ DBusConnectionPtr dbus_get_bus_connection(const char *busType,
         }
     }
 
-    if(name) {
-        // Copy name, to ensure that it remains available.
-        char *copy = g_strdup(name);
-        g_bus_own_name_on_connection(conn.get(), copy, G_BUS_NAME_OWNER_FLAGS_NONE,
-                                     NULL, GDBusNameLost, copy, g_free);
+    if (name) {
+        // Request name later in undelay(), after the caller
+        // had a chance to add objects.
+        conn.addName(name);
+        // Acting as client, need to stop when D-Bus daemon dies.
         g_dbus_connection_set_exit_on_close(conn.get(), TRUE);
     }
 
@@ -124,11 +176,6 @@ DBusConnectionPtr dbus_get_bus_connection(const std::string &address,
     return conn;
 }
 
-void dbus_bus_connection_undelay(const DBusConnectionPtr &conn)
-{
-    g_dbus_connection_start_message_processing(conn.get());
-}
-
 static void ConnectionLost(GDBusConnection *connection,
                            gboolean remotePeerVanished,
                            GError *error,
@@ -145,6 +192,12 @@ static void DestroyDisconnect(gpointer data,
     delete cb;
 }
 
+void DBusConnectionPtr::flush()
+{
+    // ignore errors
+    g_dbus_connection_flush_sync(get(), NULL, NULL);
+}
+
 void DBusConnectionPtr::setDisconnect(const Disconnect_t &func)
 {
     g_signal_connect_closure(get(),
